# coding=utf-8
from __future__ import absolute_import

# TODO: enable (currently this breaks the tests)
# from sqlalchemy.exc import SAWarning
# import warnings
# warnings.simplefilter('error', SAWarning)
