{% extends "base.html" %}

{% block main %}
  <h1>Not Found</h1>

  <p>
    {%- trans -%}
      Page or entity not found. The object you are trying to view may have been deleted.
    {%- endtrans -%}
  </p>
{% endblock %}
