<div class="form-inline">
  <div class="form-group">
    <label for={{ key.id }}:days}" class="control-label">{{ _('Days') }}</label>
    <input type="text" class="form-control" value="{{ key.days }}" name="{{ key.id }}:days"/>
  </div>

  <div class="form-group">
    <label for={{ key.id }}:hours}" class="control-label">{{ _('Hours') }}</label>
    <input type="text" class="form-control" value="{{ key.hours }}" name="{{ key.id }}:hours"/>
  </div>

  <div class="form-group">
    <label for={{ key.id }}:minutes}" class="control-label">{{ _('Minutes') }}</label>
    <input type="text" class="form-control" value="{{ key.minutes }}" name="{{ key.id }}:minutes"/>
  </div>
</div>
